# -*- coding: utf-8 -*-

from .root import root_group
from .account import account_group
from .wallet import wallet_group
from .config import config_group
from .alias import alias_group
from .funds import funds_group

cli = root_group
